#pragma once
#include "FBullCowGame.h"
#include <map>

//to make it Unreal-friendly
#define TMap std::map 
using FString = std::string;
using int32 = int;

extern int32 MaxWordLength;

int32 FBullCowGame::GetHiddenWordLength()const { return (MaxWordLength); }
int32 FBullCowGame::GetCurrentTry()const { return MyCurrentTries; }
bool FBullCowGame::IsGameWon()const { return bIsGameWon;}
int32 FBullCowGame::GetEasiestDiffficulty()const{ return EasiestDifficulty;}
int32 FBullCowGame::GetHardestDiffficulty()const { return HardestDifficulty; }


FString FBullCowGame::GetHiddenWord()const
{
	//these words must be isograms
	TMap<int32, FString> HiddenWord{ { 3,"the" },{ 4,"done" },{ 5,"money" },{ 6,"planet" },{ 7,"phantom" } };
	return HiddenWord[GetHiddenWordLength()];
}

int32 FBullCowGame::GetMaxTries()const
{
	TMap<int32, int32> WordLengthtoMaxTries{{3,4}, {4,5}, {5,5}, {6,5}, {7,5}};
	return WordLengthtoMaxTries[GetHiddenWordLength()];
}


FBullCowGame::FBullCowGame() {ResetGame();} //default constructor

void FBullCowGame::ResetGame()
{
 	MyCurrentTries = 1;
	EasiestDifficulty = 3;
	HardestDifficulty = 7;
	bIsGameWon = false;
	return;
}

void FBullCowGame::SetHiddenWord()
{
	MyHiddenWord = GetHiddenWord();
	return;
}



EGuessStatus FBullCowGame::ValidateAnswer(FString Guess)const
{
	if (!IsIsogram(Guess)) //if the guess is not an isogram
	{return EGuessStatus::Not_Isogram;}
	else if (!IsLowercase(Guess)) //if the guess is not all lowercase return an error
	{return EGuessStatus::Not_lowercase;}
	else if (Guess.length() != GetHiddenWordLength()) //if the guess length is incorrect
	{return EGuessStatus::Word_length;} //if the word length is incorrect
	else
	{return EGuessStatus::Okay;}
}

//receives a valid guess, increments turn, and returns count
FBullCowCount FBullCowGame::SubmitValidGuess(FString Guess)
{
	MyCurrentTries++;

	FBullCowCount BullCowCount;
	int32 WordLength = MyHiddenWord.length(); //assuming same length as guess


	//loop through all letters in the hidden word
	for (int32 MyHWChar = 0; MyHWChar < WordLength; MyHWChar++) {
		//compare letters against the guess
		for (int32 GChar = 0; GChar < WordLength; GChar++) {
			//if they match then
			if (Guess[GChar] == MyHiddenWord[MyHWChar])
			{//if they are in the same place
				if (MyHWChar == GChar) {//increment Bulls
					BullCowCount.Bulls++;
				}

				else {BullCowCount.Cows++;
				}
			}
		}
	}
	if (BullCowCount.Bulls == WordLength) {
		bIsGameWon = true;
	}
	else {
		bIsGameWon = false;
	}
	return BullCowCount;
}

bool FBullCowGame::IsIsogram(FString Word) const
{
	//treat 0 or 1 letter words as isograms
	if (Word.length() <= 1) { return true; }
	
	TMap<char, bool> LetterSeen;	//set up our map
	for (auto Letter:Word)	//for all letters of the word
	{
		Letter = tolower(Letter); //handle mixed case
		if (LetterSeen[Letter]) {//if the letter is in the map
			return false; //we do not have an isogram
		} else {
			LetterSeen[Letter] = true; //enter the letter in the map as seen
		}
	}
	return true; //for example in cases where /0 is entered
}

bool FBullCowGame::IsLowercase(FString Word) const
{
	for (auto Letter : Word)	//for all letters of the word
	{
		if (!islower(Letter)) {
			return false; //if the letter is uppercase, it is not 
		}
		else {
			return true; //enter the letter in the map as seen
		}
	}
}



